﻿// NicoZero_extendCommentLog.js
// ZeroWatchのコメントログを拡張(ユーザーIDを表示)
// 2012/06/03	ver 1.00
// 2012/06/09	ver 1.01	仕様変更に対応
// 2013/08/03	ver 2.00	仕様変更に対応
// 2014/02/04	ver 2.01	仕様変更に対応（NG追加後にコメント欄に戻らなかったのを修正）
// 2014/02/19	ver 2.1		userIDでソートできるように変更。nicoruCountとscore（何のscoreか不明ｗNGかな？）、postedCount（コメント数）を追加。設定画面を作成
// 2014/02/20	ver 2.11	ちょっとした修正
// 2014/02/23	var 2.12	過去ログにも対応・タブの横幅変更に対応 thx>>222
// 2014/03/04	var 2.13	jQuery1.11に対応（$(selector).live(type, fn)→$(document).on(type, selector, fn)）
//							ユーザーIDでコメント抽出時に、再度ユーザーIDをクリックした時の設定を作成（全コメント表示 / NG / なにもしない）
// 2014/03/30	var 2.14	コメント検索を可能に（設定から有効化） / ログの幅を自動調節に
// 2014/04/26	var 2.15	コマンドも表示できるように変更 / コメント検索に正規表現追加（最初と最後をスラッシュにする→例：/search\s+word/） / ログが横に長くなった際表示位置を中央に調整
// 2014/07/23	var 2.16	動画一覧を見るときに画面サイズが小さくなった動画が黒くなっていたのを修正
// 2014/08/05	var 2.17	仕様変更に対応
// 2014/09/09	var 2.18	各ユーザーのコメント数が表示されない時に、ユーザー選択したら計算されるように
// 2014/09/14	var 2.19	コメント右クリック→検索が機能してなかったので修正
// 2014/09/21	var 2.20	再生準備ができてもコメント一覧に表示されていない時があるので、「全表示」ボタンを準備完了時に表示しておく。(コメントが一覧になければ「全表示」。コメント数未表示も同様)
// 2015/01/01	var 2.21	ウィンドウサイズが小さい時に左端に隙間（8px）を開けるように変更（左端にマウスカーソルを置いてスクロール用）
// 2015/02/04	var 2.22	仕様変更に対応

jQuery(function(){
	/*
		description: 説明
		fieldName: プロパティ名
		display: 表示するか
		headerName: 見出しの表示
		position: どこに表示するか（負数は既存のものの前、絶対値が小さいほうが前）
		width: 横幅 [px]（初期値）
	*/
	var addTypeList = [
		{
			description: "そのユーザーがいくつコメントしたか",
			display: true,
			fieldName: "postedCount",
			headerName: "C",
			position: -1,
			width: 25
		}, {
			description: "ユーザーID（先頭の[P]はプレミアム会員）",
			display: true,
			fieldName: "userID",
			headerName: "ユーザーID",
			position: -2,
			width: 80
		}, {
			description: "コマンド（184は除外）",
			display: true,
			fieldName: "command",
			headerName: "コマンド",
			position: 1,
			width: 100
		}, {
			description: "「ニコる」された数",
			display: true,
			fieldName: "nicoruCount",
			headerName: "N",
			position: 2,
			width: 30
		}, {
			description: "NGのスコア（共有NGに関係）",
			display: false,
			fieldName: "score",
			headerName: "S",
			position: 3,
			width: 50
		}
	];
	
	
	var EXT_CLASS_NAME = "extendCommentLog_";
	var su = SettingUtil_pr_nl.getInstance("extendCommentLog");
	var isDebug = su.getSetting("isDebug", false);
	function debugLog(){
		if (isDebug && window.console && typeof window.console.log !== "undefined") window.console.log.apply(window.console, arguments);
	}
	// setTimeout(arguments.callee, 500);対策
	var backupComment = {};
//	debugLog("NicoZero_extendCommentLog load");
	try {
		var CommentPanelView = require("playerPanel/view/CommentPanelView");
		if (typeof CommentPanelView.COLUMNS === "undefined") throw "error";
		
		var CommentProvider = require("playerPanel/view/CommentProvider");
		var original_setComments = CommentProvider.prototype.setComments;
		if (typeof original_setComments === "undefined") throw "error";
		
		var Comments = require("comments/model/Comments");
		if (typeof Comments.prototype.merge === "undefined") throw "error";
		
		
		/*
			new require("playerPanel/view/CommentPanelView").getComments()
			new require("playerPanel/view/PlayerPanelTabsView")._commentPanelView = new CommentPanelView()
			new require("watchapp/view/player/RightSidePanelViewController")._playerPanelTabsView = new PlayerPanelTabsView()
			require("watchapp/init/PlayerInitializer").rightSidePanelViewController = new RightSidePanelViewController()
			
			require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView.getComments()
			
			var CommentPanelView = require("playerPanel/view/CommentPanelView");
			
		*/
	} catch(e) {
		return setTimeout(arguments.callee, 300);
	}
//	debugLog("NicoZero_extendCommentLog START");

	addTypeList.sort(function (a, b){
		return a.position - b.position;
	});
	/*
		require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView._grid = new require("slick.Grid")(this._$grid, this._provider, b.COLUMNS, {
			enableColumnReorder: !1,
			explicitInitialization: !0
		});
		var _grid = require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView._grid;
	*/
	
	CommentProvider.prototype.setComments = function(_comments) {
		// 同一ユーザーのコメントをカウント
		if (_comments) {
			if (!_comments.nowExtracting) {
				backupComment[_comments.name] = _comments._data;
			}
			var data = _comments._data;
			var count  = {};
			for (var i = 0, len = backupComment[_comments.name].length; i < len; i++){
				var id = backupComment[_comments.name][i].userID;
				if (typeof count[id] === "undefined") count[id] = 1;
				else count[id]++;
			}
			for (var i = 0, len = data.length; i < len; i++){
				data[i].postedCount = count[data[i].userID];
			}
			debugLog("CommentProvider.prototype.setComments", _comments.name, _comments._data.length, backupComment[_comments.name].length, _comments);
			commentCountInfo.text(_comments.length + " / " + backupComment[_comments.name].length);
		}
		return original_setComments.apply(this, arguments);
	};
	// コメント取得関数を書き換え
	var original_getItem = CommentProvider.prototype.getItem;
	CommentProvider.prototype.getItem = function(a) {
		var originalReturn = original_getItem.apply(this, arguments);
		if (originalReturn) {
			var obj = this.getComment(a);
			if (obj) {
				if (this._converter) obj = this._converter(obj);
				originalReturn.userID = (obj._data.isPremium ? "[P]" : "") + obj.userID;
	//			originalReturn.userID = (obj._data.isPremium ? "[P]" : "") + obj.userID.slice(0,8);
				// コマンドは色付きにしたい？:たしかinnerHTMLで入れてるからできるかも？
				originalReturn.command = obj._data.command.replace(/(\s|^)184(\s|$)/, "$1$2");
				originalReturn.nicoruCount = obj._data.nicoruCount;
				originalReturn.score = obj._data.score;
				originalReturn.postedCount = obj.postedCount;
			}
		}
		return originalReturn;
		/*
		if (a = this.getComment(a)) return this._converter && (a = this._converter(a)), {
			message: a.message,
			vpos: a.vpos,
			formattedVpos: moment(a.vpos).format("mm:ss"),
			postedAt: a.postedAt,
			formattedPostedAt: moment(a.postedAt).format("MM/DD HH:mm"),
			resNo: a.resNo,
			// userIDのカウントをしたい
			userID: (a._data.isPremium ? "[P]" : "") + a.userID.slice(0,8),
			// コマンドは色付きにしたい？
			command: a._data.command.replace(/(\s|^)184(\s|$)/, "$1$2"),
			nicoruCount: a._data.nicoruCount,
			score: a._data.score
		}
		//*/
	};
	// コメントmerge関数を書き換え
	var original_merge = Comments.prototype.merge;
	Comments.prototype.merge = function() {
		var name = this.name;
		// 抽出中の場合は全コメントに戻す
		this._data = backupComment[name] || this._data;
		// merge
		var originalReturn = original_merge.apply(this, arguments);
		debugLog("Comments.prototype.merge", this.name, arguments);
		
		backupComment[name] = this._data;
		
		// コメント抽出中なら抽出する
		extractComments();
		return originalReturn;
	};
	
	/************	横幅調整	************/
	var cssUtil_chipWallList = WatchApp.ns.util.CSSUtil.create();
	var cssUtil_centering = WatchApp.ns.util.CSSUtil.create();
	var firstSetCommentTableWidth = true;
	function setCommentTableWidth(){
		var newWidth = 0;
		jQuery(".slick-header-columns:first").children(":visible").each(function(){
	//		newWidth += jQuery(this).width();
			newWidth += this.offsetWidth;
		});
		debugLog("setCommentTableWidth: " + newWidth);
		if (newWidth !== 0) {
			newWidth += 30;
			document.getElementById("playerTabContainer").style.width = newWidth + "px";
			
			jQuery("#playerTabContainer .player-tab-header .player-tab-item").css("width", (newWidth / 2) + "px");
			
			var playerCommentWidth = jQuery("#playerNicoplayer").width() + newWidth;
			var paaWidth = jQuery("#playerAlignmentArea").width();
			var overWidth = playerCommentWidth - paaWidth;
			var wWidth = jQuery(window).width();
			
			var ml;
			// 横スクロールバーが出る時は左側に寄せる
			if (wWidth < playerCommentWidth) {
				ml = (wWidth - paaWidth) / 2;
				// 左側に必ず8px余白を作る（スクロール用）
				ml -= 8;
			} else {
				ml = overWidth / 2;
			}
			
			if (ml < 0) ml = 0;
			// 壁紙変更ボタン位置
			var cwlr = overWidth - ml + 40;
			cssUtil_chipWallList.set("#chipWallList", "right: -" + cwlr + "px");
			// 中央寄せ
			cssUtil_centering.set("#playerAlignmentArea:not(.size_small) #playerContainer:not(.full_with_browser)", "margin-left: -" + ml + "px !important;");
			
			if (firstSetCommentTableWidth) {
				firstSetCommentTableWidth = false;
				// 一回再描画しないと表示されない列（後方の列）がある
				jQuery(".player-tab-item.ng").click();
				jQuery(".player-tab-item.comment").click();
			}
		}
	}
	
	// ページ表示時に1回実行
	window.setTimeout(setCommentTableWidth, 1000);
	// プレイヤーサイズ変更したら実行（WatchAppの解読難しい・・・）
	WatchJsApi.player.onPlayerScreenModeChanged(function(obj){
		if (WatchJsApi.player.PlayerScreenMode.BROWSER_FULL !== obj._mode) {
			setCommentTableWidth();
		}
	});
	// ウィンドウサイズ変更（完了）時に実行
	var tId = null;
	jQuery(window).resize(function() {
		if (tId !== null) clearTimeout(tId);
		tId = setTimeout(setCommentTableWidth, 100);
	});
	// コメント幅変更完了時に実行
//	jQuery(".slick-resizable-handle").on("dragend", setCommentTableWidth);				// この行が実行時に要素が無いとイベント登録されないのでNG
//	jQuery(document).on("dragend", ".slick-resizable-handle", setCommentTableWidth);	// カスタムイベントはdocumentにlive登録できない？
	(function(){
		try {
			require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView._getGrid().onColumnsResized.subscribe(setCommentTableWidth);
			// define("playerPanel/view/withGridHeaderSizeRestore")
			return;
		} catch(e) {
		}
		setTimeout(arguments.callee, 2000);
	})();
	
	/************	表示させる列の反映	************/
	function refreshColumnDisplay(){
		// デフォのCOLUMNSをコピー
		var COLUMNS = CommentPanelView.COLUMNS.concat();
		for (var i = 0, len = addTypeList.length; i < len; i++) {
			var obj = {
				sortable: true
			};
			obj.id = obj.field = addTypeList[i].fieldName;
			obj.name = addTypeList[i].headerName;
			obj.width = addTypeList[i].width;
			obj.headerCssClass = obj.cssClass = EXT_CLASS_NAME + addTypeList[i].fieldName;
			obj.toolTip = addTypeList[i].description;
				
			addTypeList[i].display = su.getSetting(addTypeList[i].fieldName, addTypeList[i].display);
			
			if (addTypeList[i].display) {
				if (addTypeList[i].position < 0) {
					COLUMNS.unshift(obj);
				} else {
					COLUMNS.push(obj);
				}
			}
		}
		require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView._getGrid().setColumns(COLUMNS);
		setCommentTableWidth();
	}
	// ページ表示時に1回実行
	(function(){
		try {
			if (typeof require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView._getGrid().setColumns !== "function") throw "error";
			refreshColumnDisplay();
			return;
		} catch(e) {
		}
		setTimeout(arguments.callee, 300);
	})();
	/*
	var cssUtil_hideColumn = WatchApp.ns.util.CSSUtil.create();
	function refreshColumnDisplay_old(){
		// CSSでの反映はズレが発生した
		var hideColumnArray = [];
		for (var i = 0, len = addTypeList.length; i < len; i++) {
			if (addTypeList[i].hide) {
				hideColumnArray.push(EXT_CLASS_NAME + addTypeList[i].fieldName);
			}
		}
		console.log(hideColumnArray);
		if (hideColumnArray.length) {
			cssUtil_hideColumn.set("." + hideColumnArray.join(", ."), "display: none !important;");
		} else {
			cssUtil_hideColumn.clear();
		}
		setCommentTableWidth();
	}
	*/
	
	/************	設定ボタンや検索窓の表示領域	************/
	var ECL_header = jQuery("<span>").css("margin-bottom", "4px");
	jQuery(".panel-header .comment-type").after(ECL_header);
	
	// 表示（抽出）中のコメント数 / 表示可能コメント数
	var commentCountInfo = jQuery('<span>').css("margin-right", "4px").attr("title", "NGされていて表示されないコメントもカウントしてあります");
	ECL_header.append(commentCountInfo);
	
	// 全コメントを表示するボタン
	var showAllCommentButton = jQuery('<button>').text("全表示").click(showAllComment).hide();
	ECL_header.append(showAllCommentButton);
	
	// NGするボタン
	var ngCommentButton = jQuery('<button>').text("NG").click(ngComment).hide();
	ECL_header.append(ngCommentButton);
	
	/************	ユーザーID右クリック・キーワード検索でコメント抽出	************/
	var nowExtracting_userId = false;
	var nowExtracting_Search = false;
	
	(function (){
		//動画切り替え後に初期化
		try {
			WatchApp.namespace.model.player.NicoPlayerConnector.playerAreaConnector.addEventListener("onVideoInitialized", function(){
				nowExtracting_userId = false;
				nowExtracting_Search = false;
			});
			return;
		} catch(e) {}
		setTimeout(arguments.callee, 2000);
	})();
	
	jQuery(document).on("click", ".comment-panel .grid-canvas", function (evt) {
		// コメント抽出時に空白領域をクリックで全表示に切り替え
		// 本当はこれも設定あると良い？
		if (evt.target === this) {
			if (nowExtracting_userId || nowExtracting_Search) {
				showAllComment();
			}
		}
	});
	function extractComments(){
		if (nowExtracting_userId || nowExtracting_Search) {
			var _commentPanelView =require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView;
			var _comments = _commentPanelView.getComments();
			var allComment = backupComment[_comments.name];
			var _data = [];
			if (nowExtracting_userId) {
				jQuery.each(allComment, function (index, eachData){
					if (eachData.userID === extractedUserId) _data.push(eachData);
				});
				debugLog("ユーザーIDでコメント抽出: " + extractedUserId, _comments._data.length, _data.length);
				
				commentSearchBox.val("");
				ngCommentButton.text("NGユーザー").show();
			} else if (nowExtracting_Search) {
				if (extractedKeyword.match(/^[/](.*)[/]$/)) {
					var reg = new RegExp(RegExp.$1);
					jQuery.each(allComment, function (index, eachData){
						if (reg.test(eachData.message)) _data.push(eachData);
					});
					ngCommentButton.hide();
				} else {
					var word = extractedKeyword.toLowerCase();
					jQuery.each(allComment, function (index, eachData){
						if (eachData.message.toLowerCase().indexOf(word) !== -1) _data.push(eachData);
					});
					ngCommentButton.text("NGワード").show();
				}
				debugLog("キーワードでコメント抽出: " + extractedKeyword, _comments._data.length, _data.length);
			}
			
			_comments.length= _data.length;
			_comments._data = _data;
			_comments.nowExtracting = true;
			_commentPanelView._setComments(_comments);
			showAllCommentButton.show();
		}
	}
	function showAllComment(){
		// 全コメント表示
		var _commentPanelView =require("watchapp/init/PlayerInitializer").rightSidePanelViewController._playerPanelTabsView._commentPanelView;
		var _comments = _commentPanelView.getComments();
		var _data = backupComment[_comments.name];
		debugLog("全コメント表示", _comments._data.length, _data.length);
		_comments.length= _data.length;
		_comments._data = _data;
		_comments.nowExtracting = false;
		_commentPanelView._setComments(_comments);
		
		nowExtracting_userId = false;
		nowExtracting_Search = false;
		
		commentSearchBox.val("");
		showAllCommentButton.hide();
		ngCommentButton.hide();
	}
	var nowNGAdding = false;
	function ngComment(){
		if (nowExtracting_userId || nowExtracting_Search) {
			jQuery("#playerTabContainer .player-tab-item.ng").click();
			jQuery("#playerTabContainer .ng-panel-menu .ng-register-button").click();
			if (nowExtracting_userId) {
				jQuery("#playerTabContainer .ng-panel input.ng-source-input").val(extractedUserId);
				jQuery('#playerTabContainer .ng-panel .ng-type-radio[value=user]').click();
			} else if (nowExtracting_Search) {
				jQuery("#playerTabContainer .ng-panel input.ng-source-input").val(extractedKeyword);
				jQuery('#playerTabContainer .ng-panel .ng-type-radio[value=comment]').click();
			}
			// 追加
			jQuery("#playerTabContainer .ng-panel .register-button").click();
			nowNGAdding = true;
			showAllComment();
		}
	}
	(function (){
		//NG追加後に実行
		try {
			require("watchapp/init/PlayerInitializer").rightSidePanelViewController._commentNGListStore.on("updated", function(){
				debugLog("NG:updated");
				if (nowNGAdding) {
					jQuery("#playerTabContainer .player-tab-item.comment").click();
					nowNGAdding = false;
				}
			});
			return;
		} catch(e) {}
		setTimeout(arguments.callee, 2000);
	})();
	
	
	/************	キーワード検索でコメント抽出	************/
	var extractedKeyword;
	
	var useSearchBox = su.getSetting("useSearchBox", false);
	var commentSearchBox = jQuery('<input type="text" size="20" placeholder="コメントから検索して抽出" title="コメントから検索して抽出\nEnterで検索\n前後を/で囲むと正規表現検索\n例：/[0-9]+/">');
	ECL_header.append(commentSearchBox);
	if (!useSearchBox) commentSearchBox.hide();
	commentSearchBox.keydown(function(evt){
		if (evt.keyCode === 13) {	// Enter : 検索（ユーザーIDで抽出中でも全コメントから検索）
			var word = this.value.replace(/(^[\s　]+)|([\s　]+$)/g, "");
			if (word) {
				nowExtracting_userId = false;
				nowExtracting_Search = true;
				extractedKeyword = word;
				extractComments();
			} else {
				showAllComment();
			}
		} else if (evt.keyCode === 27) {	// Esc : フォーカスを外す
			this.blur();
		}
	});
	function searchComment(allComment, word){
		var arr = [];
		if (typeof word === "string") {
			word = word.toLowerCase();
			for (var i = 0, len = allComment.length; i < len; i++) if (allComment[i].message.toLowerCase().indexOf(word) !== -1) arr.push(allComment[i]);
		} else {
			// word: 正規表現
			for (var i = 0, len = allComment.length; i < len; i++) if (word.test(allComment[i].message)) arr.push(allComment[i]);
		}
		return arr;
	}
	
	/************	ユーザーID右クリックでコメント抽出	************/
	var extractedUserId;
	var userIdSecondClick_ShowAll = "ShowAll";
	var userIdSecondClick_NG = "NG";
	var userIdSecondClick_DoNothing = "DoNothing"; // 実は未使用
	var userIdSecondClick = su.getSetting("userIdSecondClick", userIdSecondClick_ShowAll);
	
	jQuery(document).on("click", ".comment-panel .grid-canvas ." + EXT_CLASS_NAME + "userID", function () {
		if (nowExtracting_userId) {
			switch (userIdSecondClick) {
				case userIdSecondClick_ShowAll: 
					showAllComment();
					break;
				case userIdSecondClick_NG: 
					ngComment();
					break;
				case userIdSecondClick_DoNothing: 
					break;
			}
		} else {
			nowExtracting_userId = true;
			nowExtracting_Search = false;
			extractedUserId = this.innerText.replace("[P]", "");
			extractComments();
		}
	});
	
	/************	設定画面の作成	************/
	function setSettingPanel() {
		try {
			// 設定画面の準備が終わる前に、このscriptが実行されてしまうので、準備完了まで待つ
			if (typeof require("watchapp/init/PlayerInitializer").playerSettingViewController.$selector === "undefined") throw "error";

			var esp = new NicoGinza_extendSettingPanel("extendCommentLog", "拡張ログ", "コメントログにユーザーIDを表示する拡張");
	//		debugLog(esp);
			if (esp.IsEnabled) {
				var DEFALUT_VALUE_TRUE = true;
				var DEFALUT_VALUE_FALSE = false;
				esp.start_SettingUtil_pr_nl();
				esp.addElement('<div class="left"></div><div class="right"></div>');
				esp.addElement('<p>・表示する情報（※変更すると列幅がリセットされます）</p>', ".left");
				for (var i = 0, len = addTypeList.length; i < len; i++) (function(arg){
					var type = arg.fieldName;
					var desc = arg.description;
					esp.addSettingElementCheckbox(type, true, desc, function(value){
						// arg.display = !value;	// これなくても自動保存されている
						refreshColumnDisplay();
					}, ".left").css("margin-left", "20px");
				})(addTypeList[i]);
				
				esp.addSettingElementCheckbox("useSearchBox", DEFALUT_VALUE_FALSE, "コメントの検索ボックスを表示", function(value){
					useSearchBox = value;
					var method = value ? "show" : "hide";
					commentSearchBox[method]();
				}, ".left")
				
				esp.addSettingElementRadioButton("userIdSecondClick", userIdSecondClick_ShowAll, "・ユーザーコメント抽出中にID部分クリック", [userIdSecondClick_ShowAll, userIdSecondClick_NG, userIdSecondClick_DoNothing], ["コメント一覧に戻る（全コメントを表示）", "そのユーザーをNG", "なにもしない"], function(value){
					userIdSecondClick = value;
				}, ".right");
				
				esp.addSettingElementCheckbox("isDebug", DEFALUT_VALUE_FALSE, "デバッグログ", function(value){
					isDebug = value;
				}, ".right").css("position", "absolute").css("bottom", "60px").css("right", "25px");
				
				jQuery('<button title="extendCommentLog（ユーザーIDを表示する拡張）の設定">設定</button>').click(function(){
					esp.show();
				}).css("margin-right", "4px").insertBefore(ECL_header.find(":first"));
			}
			return;
		} catch(e) {
		}
		setTimeout(setSettingPanel, 2000);
	}
	setSettingPanel();
	
	/************	右クリック時メニュー拡張	************/
	function nicoSearch(word, type){
		WatchApp.namespace.init.SearchBarInitializer.searchBarViewController.search(word, type);
		
		return;
		
		var _contentViews = WatchApp.namespace.init.VideoExplorerInitializer.videoExplorerView._contentListView._contentViews;
		debugLog([word, type, "WatchApp.namespace.init.VideoExplorerInitializer.videoExplorerView._contentListView._contentViews", _contentViews]);
		// 何番目に保存されるのか知らんから全部使ってみる。
		for (var key in _contentViews) {
			try {
				_contentViews[key]._searchBoxView.dispatchEvent("search", {
					searchWord: word, 
					searchType: type
				});
				break;
			} catch(e) {}
		}
	}
	var SearchType = WatchApp.namespace.components.nicosearchstatus.model.SearchType;
	// KEYWORD, TAG
	jQuery(".comment-panel .context-menu ul.menu").append(
		jQuery('<li>キーワード検索</li>').click(function(){
			nicoSearch(jQuery(".context-menu .comment").val(), SearchType.KEYWORD);
		}).append(
			jQuery("<span> (<span>選択</span>)</span>").find("span").click(function(){
				event.stopPropagation();
				var word = window.prompt("検索ワードを残してEnter(OK)でキーワード検索", jQuery(".context-menu .comment").val());
				if (!word) return;
				nicoSearch(word, SearchType.KEYWORD);
			}).hover(fn_bgc_change_on, fn_bgc_change_off).parent()
		)
	)
	.append(
		jQuery('<li>タグ検索</li>').click(function(){
			nicoSearch(jQuery(".context-menu .comment").val(), SearchType.TAG);
		}).append(
			jQuery("<span> (<span>選択</span>)</span>").find("span").click(function(){
				event.stopPropagation();
				var word = window.prompt("検索ワードを残してEnter(OK)でタグ検索", jQuery(".context-menu .comment").val());
				if (!word) return;
				nicoSearch(word, SearchType.TAG);
			}).hover(fn_bgc_change_on, fn_bgc_change_off).parent()
		)
	)
	.append(
		jQuery('<li>google検索</li>').click(function(){
			window.open("http://www.google.co.jp/search?hl=ja&q=" + encodeURIComponent(jQuery(".context-menu .comment").val()));
		}).append(
			jQuery("<span> (<span>選択</span>)</span>").find("span").click(function(event){
				event.stopPropagation();
				var word = window.prompt("検索ワードを残してEnter(OK)でgoogle検索", jQuery(".context-menu .comment").val());
				if (!word) return;
				window.open("http://www.google.co.jp/search?hl=ja&q=" + encodeURIComponent(word));
			}).hover(fn_bgc_change_on, fn_bgc_change_off).parent()
		)
	);
	function fn_bgc_change_on(){
		this.style.backgroundColor = "orange";
	}
	function fn_bgc_change_off(){
		this.style.backgroundColor = "";
	}
})
